{}
{ QfelFileNameEditPL                                                           }
{ Pole edycyjne z przyciskiem  umoliwiajcym wskazanie pliku                  }
{ (C)1998 by Marcin Qfel Zaleski                                               }
{ Based on ToolEdit.pas from RxLib 2.50 (C) 1995, 1996 AO ROSNO                }
{                                       (C) 1997, 1998 Master-Bank             }
{}
{ QfelFileNameEditPL.pas (C)1998 by Marcin Qfel Zaleski                        }
{}
{ Changes made to the original RxLib's file:                                  }
{ - new property:PanelBevelInner - changes inner bevel of TPanel component    }
{   within the dialog window                                                   }
{ - new property:PanelBevelOuter - changes outer bevel of TPanel component    }
{   within the dialog window                                                   }
{ - new property:WindowImageMargin - initial margin between TImage and inner  }
{   bevel of TScrollBox within the TPreviewWindow                              }
{ - changed routine:CreateEditDialog - added new dialog types
{ - some minor changes
{}
{ 1998-11-19 started                                                           }
{ 1998-11-21 finished                                                          }
{}

unit QfelFileNameEditPL;

interface

uses
  ToolEdit, Dialogs, Classes, ExtCtrls;

type

  TQfelFileNameEditDialogKind = (dkOpen,dkSave,dkOpenPicture,dkSavePicture,
                                           dkOpenPicturePL,dkSavePicturePL);

  TQfelFileNameEditPL = class(TFileDirEdit)
    private
      FWindowImageMargin :Integer;
      FPanelBevelInner :TPanelBevel;
      FPanelBevelOuter :TPanelBevel;
      FDialog          :TOpenDialog;
      FDialogKind      :TQfelFileNameEditDialogKind;
      FMultipleDirs    :Boolean;
      procedure CreateEditDialog;
      function GetFileName:string;
      function GetDefaultExt:TFileExt;
      function GetFileEditStyle:TFileEditStyle;
      function GetFilter:string;
      function GetFilterIndex:Integer;
      function GetInitialDir:string;
      function GetHistoryList:TStrings;
      function GetOptions:TOpenOptions;
      function GetDialogTitle:string;
      function GetDialogFiles:TStrings;
      procedure SetDialogKind(Value:TQfelFileNameEditDialogKind);
      procedure SetFileName(const Value:string);
      procedure SetDefaultExt(Value:TFileExt);
      procedure SetFileEditStyle(Value:TFileEditStyle);
      procedure SetFilter(const Value:string);
      procedure SetFilterIndex(Value:Integer);
      procedure SetInitialDir(const Value:string);
      procedure SetHistoryList(Value:TStrings);
      procedure SetOptions(Value:TOpenOptions);
      procedure SetDialogTitle(const Value:string);
      function IsCustomTitle:Boolean;
      function IsCustomFilter:Boolean;
    protected
      procedure ButtonClick; override;
      procedure ReceptFileDir(const AFileName:string); override;
      procedure ClearFileList; override;
      function GetLongName:string; override;
      function GetShortName:string; override;
    public
      constructor Create(AOwner:TComponent); override;
      property Dialog:TOpenDialog read FDialog;
      property DialogFiles:TStrings read GetDialogFiles;
    published
      property WindowImageMargin:Integer read FWindowImageMargin write FWindowImageMargin;
      property PanelBevelInner:TPanelBevel read FPanelBevelInner write FPanelBevelInner;
      property PanelBevelOuter:TPanelBevel read FPanelBevelOuter write FPanelBevelOuter;
      property MultipleDirs:Boolean read FMultipleDirs write FMultipleDirs default False;
      property DialogKind:TQfelFileNameEditDialogKind read FDialogKind write SetDialogKind default dkOpen;
      property DefaultExt:TFileExt read GetDefaultExt write SetDefaultExt;
      property FileEditStyle:TFileEditStyle read GetFileEditStyle write SetFileEditStyle default fsEdit;
      property FileName:string read GetFileName write SetFileName stored False;
      property Filter:string read GetFilter write SetFilter stored IsCustomFilter;
      property FilterIndex:Integer read GetFilterIndex write SetFilterIndex default 1;
      property InitialDir:string read GetInitialDir write SetInitialDir;
      property HistoryList:TStrings read GetHistoryList write SetHistoryList;
      property DialogOptions:TOpenOptions read GetOptions write SetOptions default [ofHideReadOnly];
      property DialogTitle:string read GetDialogTitle write SetDialogTitle stored IsCustomTitle;
      property AutoSelect;
      property ButtonHint;
      property BorderStyle;
      property CharCase;
      property ClickKey;
      property Color;
      property Ctl3D;
      property DirectInput;
      property DragCursor;
      property DragMode;
      property EditMask;
      property Enabled;
      property Font;
      property GlyphKind;
      property Glyph;
      property ButtonWidth;
      property HideSelection;
      property Anchors;
      property BiDiMode;
      property Constraints;
      property DragKind;
      property ParentBiDiMode;
      property ImeMode;
      property ImeName;
      property NumGlyphs;
      property ParentColor;
      property ParentCtl3D;
      property ParentFont;
      property ParentShowHint;
      property PopupMenu;
      property ReadOnly;
      property ShowHint;
      property TabOrder;
      property TabStop;
      property Text;
      property Visible;
      property OnChange;
      property OnClick;
      property OnDblClick;
      property OnDragDrop;
      property OnDragOver;
      property OnEndDrag;
      property OnEnter;
      property OnExit;
      property OnKeyDown;
      property OnKeyPress;
      property OnKeyUp;
      property OnMouseDown;
      property OnMouseMove;
      property OnMouseUp;
      property OnStartDrag;
      property OnEndDock;
      property OnStartDock;
  end;

procedure Register;

implementation

uses
  SysUtils, VCLUtils, FileUtil, ExtDlgs, Controls, QfelExtendedDialogsPL;


resourcestring
  SBrowse = 'Przegldaj';
  SDefaultFilter = 'Wszystkie pliki (*.*)|*.*';
  SInvalidFilename = 'Niewaciwa nazwa pliku';

function ClipFilename(const FileName:string):string;
var
  Params : string;
begin
  if FileExists(FileName) then
    Result := FileName
  else
    SplitCommandLine(FileName,Result,Params);
end;

function ExtFilename(const FileName:string):string;
begin
  if (Pos(' ',FileName) > 0) and (FileName[1] <> '"') then
    Result := Format('"%s"',[FileName])
  else
    Result := FileName;
end;

constructor TQfelFileNameEditPL.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FPanelBevelOuter := bvLowered;
  FPanelBevelInner := bvNone;
  CreateEditDialog;
end;

procedure TQfelFileNameEditPL.CreateEditDialog;
var
  NewDialog : TOpenDialog;
begin
  case FDialogKind of
    dkOpen          : NewDialog := TOpenDialog.Create(Self);
    dkOpenPicture   : NewDialog := TOpenPictureDialog.Create(Self);
    dkSavePicture   : NewDialog := TSavePictureDialog.Create(Self);
    dkOpenPicturePL : NewDialog := TQfelOpenPictureDialogPL.Create(Self);
    dkSavePicturePL : NewDialog := TQfelSavePictureDialogPL.Create(Self);
  else
      NewDialog := TSaveDialog.Create(Self);
  end;
  try
    if FDialog<>nil then
      begin
        with NewDialog do
          begin
            DefaultExt := FDialog.DefaultExt;
            FileEditStyle := FDialog.FileEditStyle;
            FileName := FDialog.FileName;
            Filter := FDialog.Filter;
            FilterIndex := FDialog.FilterIndex;
            InitialDir := FDialog.InitialDir;
            HistoryList := FDialog.HistoryList;
            Files.Assign(FDialog.Files);
            Options := FDialog.Options;
            Title := FDialog.Title;
          end;
        if NewDialog is TQfelOpenPictureDialogPL then
          begin
            if FDialog is TQfelOpenPictureDialogPL then
              begin
                TQfelOpenPictureDialogPL(NewDialog).PanelBevelInner := TQfelOpenPictureDialogPL(FDialog).PanelBevelInner;
                TQfelOpenPictureDialogPL(NewDialog).PanelBevelOuter := TQfelOpenPictureDialogPL(FDialog).PanelBevelOuter;
                TQfelOpenPictureDialogPL(NewDialog).WindowImageMargin := TQfelOpenPictureDialogPL(FDialog).WindowImageMargin;
              end
            else
              begin
                TQfelOpenPictureDialogPL(NewDialog).PanelBevelInner := FPanelBevelInner;
                TQfelOpenPictureDialogPL(NewDialog).PanelBevelOuter := FPanelBevelOuter;
                TQfelOpenPictureDialogPL(NewDialog).WindowImageMargin := FWindowImageMargin;
              end;
          end;
        FDialog.Free;
      end
    else
      begin
        with NewDialog do
          begin
            Title := SBrowse;
            Filter := SDefaultFilter;
            Options := [ofHideReadOnly];
          end;
        if NewDialog is TQfelOpenPictureDialogPL then
          begin
            TQfelOpenPictureDialogPL(NewDialog).PanelBevelInner := FPanelBevelInner;
            TQfelOpenPictureDialogPL(NewDialog).PanelBevelOuter := FPanelBevelOuter;
            TQfelOpenPictureDialogPL(NewDialog).WindowImageMargin := FWindowImageMargin;
          end;
      end;
  finally
    FDialog := NewDialog;
  end;
end;

function TQfelFileNameEditPL.IsCustomTitle:Boolean;
begin
  Result := CompareStr(SBrowse,FDialog.Title) <> 0;
end;

function TQfelFileNameEditPL.IsCustomFilter:Boolean;
begin
  Result := CompareStr(SDefaultFilter,FDialog.Filter) <> 0;
end;

procedure TQfelFileNameEditPL.ButtonClick;
var
  Temp   : string;
  Action : Boolean;
begin
  inherited ButtonClick;
  Temp := inherited Text;
  Action := True;
  Temp := ClipFilename(Temp);
  DoBeforeDialog(Temp,Action);
  if not Action then
    Exit;
  if ValidFileName(Temp) then
    try
      if DirExists(ExtractFilePath(Temp)) then
        SetInitialDir(ExtractFilePath(Temp));
      if (ExtractFileName(Temp) = '') or
        not ValidFileName(ExtractFileName(Temp)) then
          Temp := '';
      FDialog.FileName := Temp;
    except
      { ignore any exceptions }
    end;
  FDialog.HelpContext := Self.HelpContext;
  DisableSysErrors;
  try
    Action := FDialog.Execute;
  finally
    EnableSysErrors;
  end;
  if Action then
    Temp := FDialog.FileName;
  if CanFocus then
    SetFocus;
  DoAfterDialog(Temp,Action);
  if Action then
    begin
      inherited Text := ExtFilename(Temp);
      SetInitialDir(ExtractFilePath(FDialog.FileName));
    end;
end;

function TQfelFileNameEditPL.GetFileName:string;
begin
  Result := ClipFilename(inherited Text);
end;

procedure TQfelFileNameEditPL.SetFileName(const Value:string);
begin
  if (Value = '') or ValidFileName(ClipFilename(Value)) then
    begin
      inherited Text := ExtFilename(Value);
      ClearFileList;
    end
  else
    raise EComboEditError.CreateFmt(SInvalidFilename,[Value]);
end;

{$IFDEF WIN32}

function TQfelFileNameEditPL.GetLongName:string;
begin
  Result := ShortToLongFileName(FileName);
end;

function TQfelFileNameEditPL.GetShortName:string;
begin
  Result := LongToShortFileName(FileName);
end;

{$ENDIF WIN32}

procedure TQfelFileNameEditPL.ClearFileList;
begin
  FDialog.Files.Clear;
end;

procedure TQfelFileNameEditPL.ReceptFileDir(const AFileName:string);
begin
  if FMultipleDirs then
    begin
      if FDialog.Files.Count = 0 then
        SetFileName(AFileName);
      FDialog.Files.Add(AFileName);
    end
  else
    SetFileName(AFileName);
end;

function TQfelFileNameEditPL.GetDialogFiles:TStrings;
begin
  Result := FDialog.Files;
end;

function TQfelFileNameEditPL.GetDefaultExt:TFileExt;
begin
  Result := FDialog.DefaultExt;
end;

function TQfelFileNameEditPL.GetFileEditStyle:TFileEditStyle;
begin
  Result := FDialog.FileEditStyle;
end;

function TQfelFileNameEditPL.GetFilter:string;
begin
  Result := FDialog.Filter;
end;

function TQfelFileNameEditPL.GetFilterIndex:Integer;
begin
  Result := FDialog.FilterIndex;
end;

function TQfelFileNameEditPL.GetInitialDir:string;
begin
  Result := FDialog.InitialDir;
end;

function TQfelFileNameEditPL.GetHistoryList:TStrings;
begin
  Result := FDialog.HistoryList;
end;

function TQfelFileNameEditPL.GetOptions:TOpenOptions;
begin
  Result := FDialog.Options;
end;

function TQfelFileNameEditPL.GetDialogTitle:string;
begin
  Result := FDialog.Title;
end;

procedure TQfelFileNameEditPL.SetDialogKind(Value:TQfelFileNameEditDialogKind);
begin
  if FDialogKind<>Value then
    begin
      FDialogKind := Value;
      CreateEditDialog;
    end;
end;

procedure TQfelFileNameEditPL.SetDefaultExt(Value:TFileExt);
begin
  FDialog.DefaultExt := Value;
end;

procedure TQfelFileNameEditPL.SetFileEditStyle(Value:TFileEditStyle);
begin
  FDialog.FileEditStyle := Value;
end;

procedure TQfelFileNameEditPL.SetFilter(const Value:string);
begin
  FDialog.Filter := Value;
end;

procedure TQfelFileNameEditPL.SetFilterIndex(Value:Integer);
begin
  FDialog.FilterIndex := Value;
end;

procedure TQfelFileNameEditPL.SetInitialDir(const Value:string);
begin
  FDialog.InitialDir := Value;
end;

procedure TQfelFileNameEditPL.SetHistoryList(Value:TStrings);
begin
  FDialog.HistoryList := Value;
end;

procedure TQfelFileNameEditPL.SetOptions(Value:TOpenOptions);
begin
  if Value<>FDialog.Options then
    begin
      FDialog.Options := Value;
      FMultipleDirs := ofAllowMultiSelect in FDialog.Options;
      if not FMultipleDirs then
        ClearFileList;
    end;
end;

procedure TQfelFileNameEditPL.SetDialogTitle(const Value:string);
begin
  FDialog.Title := Value;
end;

procedure Register;
begin
  RegisterComponents('Qfel',[TQfelFileNameEditPL]);
end;

end.
